﻿#ifndef SYSMAINWINDOW_H
#define SYSMAINWINDOW_H

#include <QStackedWidget>
#include <topcore/topclassabs.h>
#include <QSystemTrayIcon>

class QHBoxLayout;
class QLabel;
class QPropertyAnimation;
class QStackedWidget;
class QTabBar;
class QDateTime;
class QVBoxLayout;
class TFramelessWindowBar;
class TPanelMenu;
class TopMessageController;
class TopQuickToolBar;
class TopQuickButton;
class TToolButton;
class QSplitter;

struct NaviData;

class StretchFrame: public QFrame {
    Q_OBJECT
public:
    StretchFrame(QWidget* iParent = nullptr);
    void setWidget(QWidget *iWidget);
    void setStretchWidgetVisible(bool iVisible);

protected:
    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *);
    bool event(QEvent *event);
    void showEvent(QShowEvent *event);

private:
    bool mLeftButtonPressed = false;
    QPoint mDragPosition;
    qreal mStretchWidgetWidth = 1;
    bool mMouseAnchor = false;
    QWidget *mWidget = nullptr;
    QFrame *mStretchWidget = nullptr;
    QHBoxLayout *mLayout = nullptr;
};

class StackedWidget : public QStackedWidget {
    Q_OBJECT
public:
    StackedWidget(QWidget *iParent = nullptr);
    void setBackground(const QString &iBackground);
protected:
    void paintEvent(QPaintEvent *iEvent);
private:
    QString mBackground;
};

class SysMainWindow final : public TopClassAbs
{
    Q_OBJECT
public:
    enum class RouteType {
        None,
        Module,
        Menu,
        Action,
        Url,
        ModuleBar
    };

public:
    explicit SysMainWindow(const QString &iModuleNameStr = QString(""),
                           const QVariantMap &iUrlPars = QVariantMap(),
                           QWidget *iParent = nullptr);
    ~SysMainWindow();

public slots:
    void openModule(const QString &iUrl, const QVariant &iConfig = QVariant());
    void showPasswordModificationDialog(bool iPwdStrengthCheck = true, int iPwdLengthLimit = 8, int iPwdTypeCount = 3, const QStringList &iNonPasswordLst = QStringList());
    void showAboutDialog();
    void showModuleVersionDialog();
    void showVersionDetailDialog(QWidget *iParent, const QVariant &iDetail);
    void openModuleUpgrade();
    void setIsPinned(bool iIsPinned, bool iForce = false);
    bool hide2SystemTrayState();
    void saveHide2SystemTrayState(bool iState);
    bool hide2SystemTrayTipState();
    void saveHide2SystemTrayTipState(bool iState);
    bool isSystemTrayIconShow();
    void showLicenseInfo();
    QVariantMap getLicenseInfo();
    void forAuthorization();

protected:
    bool nativeEvent(const QByteArray &iEventTypeLst,
                     void *iMsgPtr,
                     long *oResultI64Ptr) override;
    void closeEvent(QCloseEvent *iEvent) override;

private slots:
    void onOperateTimer();
    void onQuickBtnClicked();
    void onTabBarCurrentChanged(int iIndexInt);
    void onTabBarCloseRequested(int iIndexInt);
    void onNaviButtonClicked();
    void onPinButtonClicked();
    void onPanelMenuClicked(const QVariant &iDataVar);
    void onNotification(const QString &iKeyStr, const QVariant &iDataVar, const QString &iUuid);
    void onModuleWindowTitleChanged();
    void onSysTrayIconActived(QSystemTrayIcon::ActivationReason reason);

private:
    void initIsPinned();
    void initNaviToolBar();
    void initMainWidget();
    void initDefaultRoute();
    void updateTabBar(const QString &iUrl, const QVariant &iConfigVar = QVariant());
    void updateTabBarWhenRemoveTab(int iIndexInt);
    void updatePopupWidget(TopQuickButton *iBtn, const QVariantMap &iConfigVar, RouteType type);
    void movePanelMenuWidget();
    void initSysTrayIcon();
    RouteType str2RouteType(const QString &iTypeStr);
    void routeUrl(const QString &iUrlAddressStr, const QVariantMap &iConfigVar);
    void routeModule(const QString &iUrlAddressStr, const QVariantMap &iConfigVar);
    void routeAction(const QString &iUrlAddressStr);
    void route(TopQuickButton *iBtn, NaviData *iData, const QVariantMap &iConfigVar);
    void buildPanelMenuData(QVariantList &oMenuDataLst);
    QVariantMap searchDefaultRoute(const QVariantList &iNaviLst);
    void showCloseDialog();

private:
    //拼版特殊设置,超过六小时未操作,提示重新登录
    QDateTime *mPreOperateTime = nullptr;
    QPointF mPreMousePos;
    QPointF mNowMousePos;
    bool mPreWindowHideState = false;
    bool mNowWindowHideState = false;
    bool mPreWindowActiveState = false;
    bool mNowWindowActiveState = false;
    QTimer *mOperateTimeOutTimer = nullptr;

    TopQuickToolBar *mNaviToolBar = nullptr;
    TFramelessWindowBar *mWindowTitleBar = nullptr;
    QTabBar *mTabBar = nullptr;
    StackedWidget *mStackedWidget = nullptr;
    QStackedWidget *mModuleBarStackedWidget = nullptr;
    QMap<QString, TopClassAbs *> mModuleBarUrlAddressMap;
    QMap<TopQuickButton *, NaviData *> mNaviDataMap;
    TopQuickButton *mCurrentActiveQuickBtn = nullptr;

    QLabel *mPanelMenuTitleLabel = nullptr;
    TToolButton *mPinButton = nullptr;
    TPanelMenu *mPanelMenu = nullptr;
    StretchFrame *mPopupWidget = nullptr;
    QSplitter *mCenterWidget = nullptr;
    QWidget *mMainWidget = nullptr;

    QHBoxLayout *mMainLayout = nullptr;

    QMap<QString, QWidget *> mUrlAddressWidgetMap;   // urlAddress:QWidget
    QMap<QString, QVariantMap> mUrlAddressConfigMap; // urlAddress:Config

    bool mIsPinned = false;

    QLabel *mMessageLabel = nullptr;
    QPropertyAnimation *mMessageAnimation = nullptr;

    TopMessageController *mMessageController = nullptr;

    bool mIsRoutingModule = false;
    // 托盘图标
    QSystemTrayIcon *mSysTrayIcon = nullptr;
    bool mSysTrayIconShow = false; // 是否显示托盘图标
    bool mHidetoSysTray = false; // 关闭时是否隐藏程序到托盘
    bool mHidetoSysTrayFlag = false; // 注册表中是否有mHidetoSysTray标记，是否显示确认弹窗
    QString mHidetoTrayRegName;
    QString mSysTrayTitle;
    bool mModuleVersionDialogFinished = true; // 模组版本对话框是否已关闭，避免打开多个窗口
    QVariantMap mLicenseInfo;
};

#endif // SYSMAINWINDOW_H
