﻿#include "topquicktoolbar.h"
#include <QActionGroup>
#include <QMenu>
#include <QStackedWidget>
#include <QVariantMap>
#include <tbaseutil/tdataparse.h>
#include <tbaseutil/tresource.h>
#include <tbaseutil/ttheme.h>
#include <topcore/topcore.h>
#include "topquickbutton.h"

class TopQuickToolBarPrivate
{
    Q_DECLARE_PUBLIC(TopQuickToolBar)
public:
    explicit TopQuickToolBarPrivate(TopQuickToolBar *qptr)
        : languageStr(APP->language()),
          q_ptr(qptr)
    {

    }

    ~TopQuickToolBarPrivate()
    {

    }

    QString languageStr;
    QActionGroup *mToolBarActions;

protected:
    TopQuickToolBar * const q_ptr;
};

TopQuickToolBar::TopQuickToolBar(QWidget *parent)
    : QToolBar(parent),
      d_ptr(new TopQuickToolBarPrivate(this))
{
    Q_D(TopQuickToolBar);
    d->mToolBarActions = new QActionGroup(this);
   // this->setIconSize(QSize(16, 16));
}

TopQuickToolBar::~TopQuickToolBar()
{
    //  保存 来自用户配置的数据
    QEventLoop loop;
    connect(this, SIGNAL(userDataSaved()), &loop, SLOT(quit()));
    QTimer::singleShot(300, &loop, SLOT(quit()));
    emit saveUserData();
    loop.exec();
}

void TopQuickToolBar::setLanguage(const QString &iLangStr)
{
    Q_D(TopQuickToolBar);
    d->languageStr = iLangStr;
}

QString TopQuickToolBar::language() const
{
    Q_D(const TopQuickToolBar);
    return d->languageStr;
}

void TopQuickToolBar::loadQuickButtons(const QVariantList &iVarLst)
{
    for (QVariant var: iVarLst) {
        addQuickButton(var.toMap());
    }
}

TopQuickButton *TopQuickToolBar::addQuickButton(const QVariantMap &iParamMap)
{
    Q_D(TopQuickToolBar);
    TopQuickButton *btn = createQuickBtn(iParamMap);
    btn->setToolButtonStyle(Qt::ToolButtonIconOnly);
    d->mToolBarActions->addAction(addWidget(btn));
    return btn;
}

void TopQuickToolBar::clearAllButtons()
{
    Q_D(TopQuickToolBar);
    // 清除当前ACTION，会自动析构对应的QToolButoon
    QList<QAction *> listAction = d->mToolBarActions->actions();
    for (QAction *act: listAction) {
        d->mToolBarActions->removeAction(act);
        this->removeAction(act);
        act->deleteLater();
    }
}

TopQuickButton *TopQuickToolBar::createQuickBtn(const QVariantMap &iParamMap)
{
    Q_D(TopQuickToolBar);
    TopQuickButton *toolBtn = new TopQuickButton(this);
    QString btnTextStr = iParamMap.value("title_" + d->languageStr).toString();
    if (btnTextStr.isEmpty()) {
        btnTextStr = iParamMap.value("title").toString();
    }

    QString btnTooltipStr = iParamMap.value("remark_" + d->languageStr).toString();
    if (btnTooltipStr.isEmpty()) {
        btnTooltipStr = iParamMap.value("remark").toString();
    }
    if (btnTooltipStr.isEmpty()) {
        btnTooltipStr = btnTextStr;
    }

    QString iconStr = iParamMap.value("icon").toString();
    if (!iconStr.isEmpty() && !iconStr.contains('.')) {
        iconStr += ".white";
    }

    toolBtn->setText(btnTextStr);
    toolBtn->setToolTip(btnTooltipStr);
    if (!iconStr.isEmpty()) {
        toolBtn->setIcon(TRES->colorIcon(iconStr));
    }

    QString bgColor;
    if (iParamMap.contains("icon_bgcolor") &&
            !iParamMap.value("icon_bgcolor").toString().isEmpty()) {
       bgColor = iParamMap.value("icon_bgcolor").toString();
    } else {
        bgColor = TTHEME->value("COLOR_PRIMARY_5").toString();
    }

    toolBtn->setStyleSheet(QString("TopQuickButton{border:none; background:%1;margin:%2px;  border-radius:%3px;color:#FFFFFF;}\n" \
                          "TopQuickButton:hover{border-radius:%4px}\n" \
                          "QToolButton::menu-indicator{image:None;}")
                  .arg(bgColor).arg(TTHEME_DP(4)).arg(TTHEME_DP(10)).arg(TTHEME_DP(16)));
    toolBtn->setData(iParamMap);

    return toolBtn;
}
